/**
 * @file ntmrand.c
 * @author Shinichiro Nakamura
 * @brief NTMが使用する乱数生成器の実装。
 */

/*
 * ===============================================================
 *  Natural Tiny Monitor (NT-Monitor)
 * ===============================================================
 * Copyright (c) 2011-2012 Shinichiro Nakamura
 * Inspired by M, Murakami
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ===============================================================
 */

#include "ntmrand.h"

/**
 * @brief ビットポジションを得る。
 *
 * @param N ビット。
 *
 * @return ビットポジション。
 */
#define BP(N)   (1 << ((N) - 1))

/**
 * @brief 乱数生成モジュールを初期化する。
 * @details
 * この乱数生成モジュールは24ビットM系列信号によって乱数を生成する。
 *
 * @param p 乱数生成用ハンドラ。
 * @param seed 乱数の種。(0以外の値を指定しなければならない)
 *
 * @return 0 成功。
 * @return !0 失敗。
 */
int ntmrand_init(ntmrand_t *p, const uint32_t seed)
{
    if (seed == 0) {
        return -1;
    }

    p->seed = seed;
    p->reg = seed;
    return 0;
}

/**
 * @brief 乱数生成モジュールから1ビットの乱数を取り出す。
 * @details
 * この関数が乱数生成モジュールの心臓部分である。
 * 24ビットM系列信号を発生させるモジュールである。
 *
 * @param p 乱数生成用ハンドラ。
 *
 * @return 1ビットの乱数。
 */
static int ntmrand_update(ntmrand_t *p)
{
    const uint32_t mask = BP(1) | BP(3) | BP(4);
    const uint32_t bits = BP(24);
    if ((p->reg & bits) == bits) {
        p->reg = ((p->reg ^ mask) << 1) | 1;
        return 1;
    } else {
        p->reg = p->reg << 1;
        return 0;
    }
}

/**
 * @brief 乱数を取り出す。
 * @details
 * このインターフェースは、指定したビット幅に収まる値を返す。
 *
 * @param p 乱数生成用ハンドラ。
 * @param nbits 生成する乱数のビット幅。
 * @param value 値格納先。
 *
 * @return 0 成功。
 * @return !0 失敗。
 */
int ntmrand_pop(ntmrand_t *p, const uint8_t nbits, uint32_t *value)
{
    int i;

    /*
     * 値を初期化する。
     */
    *value = 0;

    /*
     * 不正なパラメータの場合、何もしない。
     */
    if (nbits == 0) {
        return -1;
    }

    /*
     * nbitsの幅を持つ値を生成する。
     */
    for (i = 0; i < nbits; i++) {
        *value |= ntmrand_update(p) ? (1 << i) : 0;
    }
    return 0;
}

